@@ -38,7 +38,6 @@
 #include <sys/stat.h>
 
 #include <bluetooth/bluetooth.h>
-#include <bluetooth/uuid.h>
 
 #include <glib.h>
 
@@ -48,21 +47,22 @@
 
 #include "log.h"
 
+#include "lib/uuid.h"
 #include "hcid.h"
 #include "sdpd.h"
 #include "adapter.h"
 #include "dbus-common.h"
 #include "agent.h"
-#include "manager.h"
-#include "mgmt.h"
+#include "profile.h"
 #include "systemd.h"
 
 #define BLUEZ_NAME "org.bluez"
 
-#define DEFAULT_AUTO_CONNECT_TIMEOUT  60 /* 60 seconds */
 #define DEFAULT_PAIRABLE_TIMEOUT       0 /* disabled */
 #define DEFAULT_DISCOVERABLE_TIMEOUT 180 /* 3 minutes */
 
+#define SHUTDOWN_GRACE_SECONDS 10
+
 struct main_opts main_opts;
 
 static const char * const supported_options[] = {
@@ -218,8 +218,7 @@ static void parse_config(GKeyFile *config)
 	} else {
 		DBG("name=%s", str);
 		g_free(main_opts.name);
-		main_opts.name = g_strdup(str);
-		g_free(str);
+		main_opts.name = str;
 	}
 
 	str = g_key_file_get_string(config, "General", "Class", &err);
@@ -267,24 +266,40 @@ static void parse_config(GKeyFile *config)
 
 static void init_defaults(void)
 {
+	uint8_t major, minor;
+
 	/* Default HCId settings */
 	memset(&main_opts, 0, sizeof(main_opts));
-	main_opts.name = g_strdup("BlueZ");
+	main_opts.name = g_strdup_printf("BlueZ %s", VERSION);
 	main_opts.class = 0x000000;
-	main_opts.autoto = DEFAULT_AUTO_CONNECT_TIMEOUT;
 	main_opts.pairto = DEFAULT_PAIRABLE_TIMEOUT;
 	main_opts.discovto = DEFAULT_DISCOVERABLE_TIMEOUT;
 	main_opts.reverse_sdp = TRUE;
 	main_opts.name_resolv = TRUE;
 	main_opts.debug_keys = FALSE;
 
-	if (gethostname(main_opts.host_name,
-				sizeof(main_opts.host_name) - 1) < 0)
-		strcpy(main_opts.host_name, "noname");
+	if (sscanf(VERSION, "%hhu.%hhu", &major, &minor) != 2)
+		return;
+
+        main_opts.did_source = 0x0002;		/* USB */
+        main_opts.did_vendor = 0x1d6b;		/* Linux Foundation */
+        main_opts.did_product = 0x0246;		/* BlueZ */
+        main_opts.did_version = (major << 8 | minor);
 }
 
 static GMainLoop *event_loop;
 
+void btd_exit(void)
+{
+	g_main_loop_quit(event_loop);
+}
+
+static gboolean quit_eventloop(gpointer user_data)
+{
+	btd_exit();
+	return FALSE;
+}
+
 static gboolean signal_handler(GIOChannel *channel, GIOCondition cond,
 							gpointer user_data)
 {
@@ -307,7 +322,11 @@ static gboolean signal_handler(GIOChannel *channel, GIOCondition cond,
 	case SIGTERM:
 		if (__terminated == 0) {
 			info("Terminating");
-			g_main_loop_quit(event_loop);
+			g_timeout_add_seconds(SHUTDOWN_GRACE_SECONDS,
+							quit_eventloop, NULL);
+
+			sd_notify(0, "STATUS=Powering down");
+			adapter_shutdown();
 		}
 
 		__terminated = 1;
@@ -315,9 +334,6 @@ static gboolean signal_handler(GIOChannel *channel, GIOCondition cond,
 	case SIGUSR2:
 		__btd_toggle_debug();
 		break;
-	case SIGPIPE:
-		/* ignore */
-		break;
 	}
 
 	return TRUE;
@@ -334,7 +350,6 @@ static guint setup_signalfd(void)
 	sigaddset(&mask, SIGINT);
 	sigaddset(&mask, SIGTERM);
 	sigaddset(&mask, SIGUSR2);
-	sigaddset(&mask, SIGPIPE);
 
 	if (sigprocmask(SIG_BLOCK, &mask, NULL) < 0) {
 		perror("Failed to set signal mask");
@@ -368,6 +383,7 @@ static gchar *option_noplugin = NULL;
 static gboolean option_compat = FALSE;
 static gboolean option_detach = TRUE;
 static gboolean option_version = FALSE;
+static gboolean option_experimental = FALSE;
 
 static void free_options(void)
 {
@@ -453,6 +469,8 @@ static GOptionEntry options[] = {
 				"Specify plugins not to load", "NAME,..." },
 	{ "compat", 'C', 0, G_OPTION_ARG_NONE, &option_compat,
 				"Provide deprecated command line interfaces" },
+	{ "experimental", 'E', 0, G_OPTION_ARG_NONE, &option_experimental,
+				"Enable experimental interfaces" },
 	{ "nodetach", 'n', G_OPTION_FLAG_REVERSE,
 				G_OPTION_ARG_NONE, &option_detach,
 				"Run with logging in foreground" },
@@ -467,10 +485,10 @@ int main(int argc, char *argv[])
 	GError *err = NULL;
 	uint16_t sdp_mtu = 0;
 	uint32_t sdp_flags = 0;
+	int gdbus_flags = 0;
 	GKeyFile *config;
 	guint signal, watchdog;
 	const char *watchdog_usec;
-	int mgmt_err;
 
 	init_defaults();
 
@@ -512,11 +530,19 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 
-	if (!manager_init("/")) {
-		error("Can't register manager interface");
+	if (adapter_init() < 0) {
+		error("Adapter handling initialization failed");
 		exit(1);
 	}
 
+	btd_agent_init();
+	btd_profile_init();
+
+	if (option_experimental)
+		gdbus_flags = G_DBUS_FLAG_ENABLE_EXPERIMENTAL;
+
+	g_dbus_set_flags(gdbus_flags);
+
 	if (option_compat == TRUE)
 		sdp_flags |= SDP_SERVER_COMPAT;
 
@@ -531,12 +557,6 @@ int main(int argc, char *argv[])
 	/* no need to keep parsed option in memory */
 	free_options();
 
-	mgmt_err = mgmt_setup();
-	if (mgmt_err < 0) {
-		error("mgmt setup failed: %s", strerror(-mgmt_err));
-		exit(1);
-	}
-
 	rfkill_init();
 
 	DBG("Entering main loop");
@@ -566,7 +586,10 @@ int main(int argc, char *argv[])
 
 	plugin_cleanup();
 
-	manager_cleanup("/");
+	btd_profile_cleanup();
+	btd_agent_cleanup();
+
+	adapter_cleanup();
 
 	rfkill_exit();
 
@@ -577,8 +600,6 @@ int main(int argc, char *argv[])
 	if (config)
 		g_key_file_free(config);
 
-	mgmt_cleanup();
-
 	disconnect_dbus();
 
 	info("Exit");
